"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setup = setup;
exports.userEventApis = void 0;

var _clear = require("./clear");

var _click = require("./click");

var _document = require("./document");

var _hover = require("./hover");

var _keyboard = require("./keyboard");

var _paste = require("./paste");

var _pointer = require("./pointer");

var _selectOptions = require("./selectOptions");

var _tab = require("./tab");

var _type = require("./type");

var _upload = require("./upload");

const userEventApis = {
  clear: _clear.clear,
  click: _click.click,
  dblClick: _click.dblClick,
  deselectOptions: _selectOptions.deselectOptions,
  hover: _hover.hover,
  keyboard: _keyboard.keyboard,
  paste: _paste.paste,
  pointer: _pointer.pointer,
  selectOptions: _selectOptions.selectOptions,
  tab: _tab.tab,
  tripleClick: _click.tripleClick,
  type: _type.type,
  unhover: _hover.unhover,
  upload: _upload.upload
};
exports.userEventApis = userEventApis;

/**
 * Start a "session" with userEvent.
 * All APIs returned by this function share an input device state and a default configuration.
 */
function setup(options = {}) {
  var _options$document;

  (0, _document.prepareDocument)((_options$document = options.document) != null ? _options$document : document);
  return _setup(options, {
    keyboardState: (0, _keyboard.createKeyboardState)(),
    pointerState: (0, _pointer.createPointerState)()
  });
}

function _setup({
  applyAccept,
  autoModify,
  delay = 0,
  document,
  keyboardMap,
  pointerMap,
  skipAutoClose,
  skipClick,
  skipHover,
  skipPointerEventsCheck = false
}, {
  keyboardState,
  pointerState
}) {
  const keyboardDefaults = {
    autoModify,
    delay,
    document,
    keyboardMap
  };
  const pointerDefaults = {
    skipPointerEventsCheck
  };
  const pointerApiDefaults = {
    delay,
    pointerMap
  };
  const clickDefaults = {
    skipHover
  };
  const typeDefaults = {
    delay,
    skipAutoClose,
    skipClick
  };
  const uploadDefaults = {
    applyAccept
  };
  const userEvent = {
    clear: (...args) => {
      return _clear.clear.call(userEvent, ...args);
    },
    click: (...args) => {
      args[1] = { ...pointerDefaults,
        ...clickDefaults,
        ...args[1]
      };
      return _click.click.call(userEvent, ...args);
    },
    dblClick: (...args) => {
      args[1] = { ...pointerDefaults,
        ...clickDefaults,
        ...args[1]
      };
      return _click.dblClick.call(userEvent, ...args);
    },
    deselectOptions: (...args) => {
      args[2] = { ...pointerDefaults,
        ...args[2]
      };
      return _selectOptions.deselectOptions.call(userEvent, ...args);
    },
    hover: (...args) => {
      args[1] = { ...pointerDefaults,
        ...args[1]
      };
      return _hover.hover.call(userEvent, ...args);
    },
    // keyboard needs typecasting because of the overloading
    keyboard: (...args) => {
      args[1] = { ...keyboardDefaults,
        ...args[1],
        keyboardState
      };
      const ret = (0, _keyboard.keyboard)(...args);

      if (ret instanceof Promise) {
        return ret.then(() => undefined);
      }
    },
    paste: (...args) => {
      return _paste.paste.call(userEvent, ...args);
    },
    // pointer needs typecasting because of the overloading
    pointer: (...args) => {
      args[1] = { ...pointerApiDefaults,
        ...args[1],
        pointerState,
        keyboardState
      };
      const ret = (0, _pointer.pointer)(...args);

      if (ret instanceof Promise) {
        return ret.then(() => undefined);
      }
    },
    selectOptions: (...args) => {
      args[2] = { ...pointerDefaults,
        ...args[2]
      };
      return _selectOptions.selectOptions.call(userEvent, ...args);
    },
    setup: options => {
      return _setup({ ...keyboardDefaults,
        ...pointerDefaults,
        ...clickDefaults,
        ...options
      }, {
        keyboardState,
        pointerState
      });
    },
    tab: (...args) => {
      return _tab.tab.call(userEvent, ...args);
    },
    tripleClick: (...args) => {
      return _click.tripleClick.call(userEvent, ...args);
    },
    // type needs typecasting because of the overloading
    type: (...args) => {
      args[2] = { ...typeDefaults,
        ...args[2]
      };
      return _type.type.call(userEvent, ...args);
    },
    unhover: (...args) => {
      args[1] = { ...pointerDefaults,
        ...args[1]
      };
      return _hover.unhover.call(userEvent, ...args);
    },
    upload: (...args) => {
      args[3] = { ...uploadDefaults,
        ...args[3]
      };
      return _upload.upload.call(userEvent, ...args);
    }
  };
  return userEvent;
}