"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getInputRange = getInputRange;
exports.hasOwnSelection = hasOwnSelection;
exports.modifySelection = modifySelection;
exports.setSelection = setSelection;
exports.setSelectionRange = setSelectionRange;
exports.updateSelectionOnFocus = updateSelectionOnFocus;

var _isElementType = require("../misc/isElementType");

var _document = require("../../document");

var _isEditable = require("../edit/isEditable");

var _isContentEditable = require("../edit/isContentEditable");

/**
 * Backward-compatible selection.
 *
 * Handles input elements and contenteditable if it only contains a single text node.
 */
function setSelectionRange(element, anchorOffset, focusOffset) {
  var _element$firstChild;

  if (hasOwnSelection(element)) {
    return setSelection({
      focusNode: element,
      anchorOffset,
      focusOffset
    });
  }
  /* istanbul ignore else */


  if ((0, _isContentEditable.isContentEditable)(element) && ((_element$firstChild = element.firstChild) == null ? void 0 : _element$firstChild.nodeType) === 3) {
    return setSelection({
      focusNode: element.firstChild,
      anchorOffset,
      focusOffset
    });
  }
  /* istanbul ignore next */


  throw new Error('Not implemented. The result of this interaction is unreliable.');
}
/**
 * Determine if the element has its own selection implementation
 * and does not interact with the Document Selection API.
 */


function hasOwnSelection(node) {
  return isElement(node) && ((0, _isElementType.isElementType)(node, 'textarea') || (0, _isElementType.isElementType)(node, 'input') && node.type in _isEditable.editableInputTypes);
}

function isElement(node) {
  return node.nodeType === 1;
}
/**
 * Determine which selection logic and selection ranges to consider.
 */


function getTargetTypeAndSelection(node) {
  const element = getElement(node);

  if (element && hasOwnSelection(element)) {
    return {
      type: 'input',
      selection: (0, _document.getUISelection)(element)
    };
  }

  const selection = element == null ? void 0 : element.ownerDocument.getSelection(); // It is possible to extend a single-range selection into a contenteditable.
  // This results in the range acting like a range outside of contenteditable.

  const isCE = (0, _isContentEditable.getContentEditable)(node) && (selection == null ? void 0 : selection.anchorNode) && (0, _isContentEditable.getContentEditable)(selection.anchorNode);
  return {
    type: isCE ? 'contenteditable' : 'default',
    selection
  };
}

function getElement(node) {
  return node.nodeType === 1 ? node : node.parentElement;
}
/**
 * Reset the Document Selection when moving focus into an element
 * with own selection implementation.
 */


function updateSelectionOnFocus(element) {
  const selection = element.ownerDocument.getSelection();
  /* istanbul ignore if */

  if (!(selection != null && selection.focusNode)) {
    return;
  } // If the focus moves inside an element with own selection implementation,
  // the document selection will be this element.
  // But if the focused element is inside a contenteditable,
  // 1) a collapsed selection will be retained.
  // 2) other selections will be replaced by a cursor
  //  2.a) at the start of the first child if it is a text node
  //  2.b) at the start of the contenteditable.


  if (hasOwnSelection(element)) {
    const contenteditable = (0, _isContentEditable.getContentEditable)(selection.focusNode);

    if (contenteditable) {
      if (!selection.isCollapsed) {
        var _contenteditable$firs;

        const focusNode = ((_contenteditable$firs = contenteditable.firstChild) == null ? void 0 : _contenteditable$firs.nodeType) === 3 ? contenteditable.firstChild : contenteditable;
        selection.setBaseAndExtent(focusNode, 0, focusNode, 0);
      }
    } else {
      selection.setBaseAndExtent(element, 0, element, 0);
    }
  }
}
/**
 * Get the range that would be overwritten by input.
 */


function getInputRange(focusNode) {
  const typeAndSelection = getTargetTypeAndSelection(focusNode);

  if (typeAndSelection.type === 'input') {
    return typeAndSelection.selection;
  } else if (typeAndSelection.type === 'contenteditable') {
    var _typeAndSelection$sel;

    // Multi-range on contenteditable edits the first selection instead of the last
    return (_typeAndSelection$sel = typeAndSelection.selection) == null ? void 0 : _typeAndSelection$sel.getRangeAt(0);
  }
}
/**
 * Extend/shrink the selection like with Shift+Arrows or Shift+Mouse
 */


function modifySelection({
  focusNode,
  focusOffset
}) {
  var _focusNode$ownerDocum, _focusNode$ownerDocum2;

  const typeAndSelection = getTargetTypeAndSelection(focusNode);

  if (typeAndSelection.type === 'input') {
    return (0, _document.setUISelection)(focusNode, {
      anchorOffset: typeAndSelection.selection.anchorOffset,
      focusOffset
    }, 'modify');
  }

  (_focusNode$ownerDocum = focusNode.ownerDocument) == null ? void 0 : (_focusNode$ownerDocum2 = _focusNode$ownerDocum.getSelection()) == null ? void 0 : _focusNode$ownerDocum2.extend(focusNode, focusOffset);
}
/**
 * Set the selection
 */


function setSelection({
  focusNode,
  focusOffset,
  anchorNode = focusNode,
  anchorOffset = focusOffset
}) {
  var _anchorNode$ownerDocu, _anchorNode$ownerDocu2;

  const typeAndSelection = getTargetTypeAndSelection(focusNode);

  if (typeAndSelection.type === 'input') {
    return (0, _document.setUISelection)(focusNode, {
      anchorOffset,
      focusOffset
    });
  }

  (_anchorNode$ownerDocu = anchorNode.ownerDocument) == null ? void 0 : (_anchorNode$ownerDocu2 = _anchorNode$ownerDocu.getSelection()) == null ? void 0 : _anchorNode$ownerDocu2.setBaseAndExtent(anchorNode, anchorOffset, focusNode, focusOffset);
}